package com.chelvc.framework.security.interceptor;

import java.io.IOException;

import com.chelvc.framework.base.util.StringUtils;
import com.chelvc.framework.security.annotation.Encrypt;
import com.chelvc.framework.security.context.SecurityContextHolder;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import lombok.extern.slf4j.Slf4j;

/**
 * 敏感数据加密序列化处理器
 *
 * @author Woody
 * @date 2023/7/28
 */
@Slf4j
public class JacksonEncryptSerializer extends StdSerializer<String> implements ContextualSerializer {
    /**
     * 敏感字段加密注解实例
     */
    private final Encrypt encrypt;

    public JacksonEncryptSerializer() {
        this(null);
    }

    public JacksonEncryptSerializer(Encrypt encrypt) {
        super(String.class);
        this.encrypt = encrypt;
    }

    @Override
    public void serialize(String plaintext, JsonGenerator generator, SerializerProvider serializer) throws IOException {
        if (this.encrypt == null || StringUtils.isEmpty(plaintext)) {
            generator.writeString(plaintext);
        } else {
            try {
                generator.writeString(SecurityContextHolder.encrypt(plaintext));
            } catch (Exception e) {
                // 如果加密失败则回退到明文传输
                log.error("Data encrypt failed: {}", e.getMessage());
                generator.writeString(plaintext);
            }
        }
    }

    @Override
    public JsonSerializer<?> createContextual(SerializerProvider serializer, BeanProperty property) {
        if (property == null) {
            return serializer.getDefaultNullValueSerializer();
        } else if (property.getType().getRawClass() != String.class) {
            return this;
        }
        Encrypt encrypt = property.getAnnotation(Encrypt.class);
        return encrypt == null ? this : new JacksonEncryptSerializer(encrypt);
    }
}
